
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: BigWigs, BigWigsAnchor, BigWigsAPI, IsAddOnLoaded, Mixin

local UIParent = UIParent

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration BigWigs")

local BAR_SPACING = 0
local BAR_PADDING = C.BORDER_PADDING
local BAR_HEIGHT = 25
local ICON_SIZE = BAR_HEIGHT

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

-- Apply SixUI's style to a BigWigs bar.
function M.ApplyStyle(bar)
	local texture = bar:GetIcon()

	B:CreateBorder(bar, BAR_PADDING, BAR_PADDING, BAR_PADDING, ICON_SIZE + BAR_PADDING)

	bar:SetHeight(BAR_HEIGHT)
	bar:SetTexture([[Interface\AddOns\SixUI\Resources\Textures\Light.tga]])
	bar.candyBarBackground:SetAlpha(0)
	bar.candyBarLabel:ClearAllPoints()
	bar.candyBarLabel:SetPoint("LEFT", bar, "LEFT", 6, 0)
	bar.candyBarDuration:ClearAllPoints()
	bar.candyBarDuration:SetPoint("RIGHT", bar, "RIGHT", -6, 0)

	if texture then
		local icon = bar.candyBarIconFrame

		bar:SetIcon(nil)
		icon:SetTexture(texture)
		icon:ClearAllPoints()
		icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT")
		icon:SetSize(ICON_SIZE, ICON_SIZE)
		icon:Show()
		bar:Set("bigwigs:restoreicon", texture)
	end
end

function M.GetSpacing(bar)
	return BAR_SPACING + (BAR_PADDING * 2)
end

function M.GetStyleName()
	return "SixUI"
end

-- Remove SixUI's style from a BigWigs bar.
function M.RemoveStyle(bar)
	local texture = bar:Get("bigwigs:restoreicon")

	B:RemoveBorder(bar)

	bar.candyBarBackground:SetAlpha(1)
	bar.candyBarLabel:ClearAllPoints()
	bar.candyBarLabel:SetPoint("TOPLEFT", bar.candyBarBar, "TOPLEFT", 2, 0)
	bar.candyBarLabel:SetPoint("BOTTOMRIGHT", bar.candyBarBar, "BOTTOMRIGHT", -2, 0)
	bar.candyBarDuration:ClearAllPoints()
	bar.candyBarDuration:SetPoint("TOPLEFT", bar.candyBarBar, "TOPLEFT", 2, 0)
	bar.candyBarDuration:SetPoint("BOTTOMRIGHT", bar.candyBarBar, "BOTTOMRIGHT", -2, 0)

	if texture then
		bar:SetIcon(texture)
		bar:Set("bigwigs:restoreicon", nil)
	end
end

function M:OnInitialize()
	if not IsAddOnLoaded("BigWigs_Plugins") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	self:RegisterStyle()
	self:ConfigureDefaults()
end

-- Configure BigWigs' default options.
function M:ConfigureDefaults()
	if (not BigWigs) then return end
	local Bars = BigWigs:GetPlugin("Bars")
	if (not Bars) then return end

	Mixin(Bars.defaultDB, {
		barStyle = self.GetStyleName(),
		BigWigsAnchor_width = 250,
		BigWigsEmphasizeAnchor_width = 250,
		emphasizeMove = false,
		emphasizeMultiplier = 1,
		fontSize = 12,
		fontSizeEmph = 12,
		growup = true,
	})

	if not self:IsHooked(BigWigsAnchor, "RefixPosition") then
		self:RawHook(BigWigsAnchor, "RefixPosition", function (...) M:OnRefixPosition(Bars.db.profile, ...) end)
	end
end

-- Register SixUI's style with BigWigs.
function M:RegisterStyle()
	if (not BigWigs) then return end
	if (not BigWigsAPI) then return end

	BigWigsAPI:RegisterBarStyle("SixUI", {
		apiVersion = 1,
		ApplyStyle = M.ApplyStyle,
		BarStopped = M.RemoveStyle,
		fontSizeEmphasized = 12,
		fontSizeNormal = 12,
		GetSpacing = M.GetSpacing,
		GetStyleName = M.GetStyleName,
		version = 1,
	})
end

M:RegisterStyle()
M:ConfigureDefaults()

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:OnRefixPosition(db, display)
	self.hooks[BigWigsAnchor].RefixPosition(display)
	if (db[display.x] and db[display.x]) then return end
	display:ClearAllPoints()
	display:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", BAR_PADDING + ICON_SIZE + 20, BAR_PADDING + 20)
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "BigWigs_Plugins" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnInitialize()
	end
end
